<?php

/**
 *
 * @copyright 2011, (c) dot2code Technologies S.L.
 *
 */

/**
 * IMPLEMENTATION NOTES:
 * 
 *   - We are not implementing the QR code removal hook (d2c_core_delete_code_hook). Removed
 *     codes are lazily detached from entities on field loading.
 *     
 *   - Removal of QR code fields or entities with QR code fields will result in QR codes not
 *     attached to any node.
 *     
 *   - The set of behaviours supported when creating QR codes through a field is a subset of all
 *     the exisiting behaviours and customization options. We would like to do some refactoring
 *     before adding all the stuff here.
 */


/**
 * 
 * 
 * Field Type API hooks.
 * 
 * 
 */


/**
 * Implements hook_field_info().
 */
function d2c_core_field_info() {
  return array(
    'd2c_core_code' => array(
      'label' => t('QR code'),
      'description' => t('Add a QR code linking to any content.'),
      'instance_settings' => array('code_creation' => 'auto_entity',
                                   'allow_code_label' => TRUE,
                                   'allow_code_visibility' => TRUE,
                                   'allow_code_detaching' => FALSE,
                                   'new' => array(
                                     'allow_code_name' => FALSE,
                                     'allow_code_description' => FALSE,
                                     'code_domain' => 'default',
                                     'code_campaign' => 'none',
                                     'code_type' => D2CAPI::CODE_TYPE_REUSABLE), 
                                   'formatter' => array(
                                     'show_label' => TRUE,
                                     'show_download_links' => TRUE,
                                     'code_width' => d2c_core_settings_get_max_code_width())
                                   ),
      'default_widget' => 'd2c_core_code_simple_field_widget',
      'default_formatter' => 'd2c_core_code_simple_field_formatter',
    ),
  );
}

/**
 * Implements hook_field_instance_settings_form().
 */
function d2c_core_field_instance_settings_form($field, $instance) {
  $settings = $instance['settings'];

  // QR code creation.
  $code_creation = array(
    'auto_entity' => t('Automatic creation of new QR codes linking to the entity (node,user, etc.) (\'node redirection\' or \'URL redirection\' behaviours)'),
    'auto_custom' => t('Automatic creation of new QR codes with \'custom\' behaviour'),
    'selection' => t('Selection and attaching of existing QR codes'),
    'manual' => t('Manual creation of new QR codes linking to any content (configurable behaviour)'),
  );
  $form['code_creation'] = array(
    '#type' => 'select',
    '#title' => t('QR code creation'),
    '#default_value' => $settings['code_creation'],
    '#required' => TRUE,
    '#options' => $code_creation,
    '#weight' => 100,
  );

  // Allow label.
  $form['allow_code_label'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow edition of labels of attached QR codes'),
    '#default_value' => $settings['allow_code_label'],
    '#description' => t('If not allowed, a default value will be generated based on the entity title.'),
    '#weight' => 101,
  );
  
  // Allow visibility.
  $form['allow_code_visibility'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow enabling/disabling visibility of attached QR codes'),
    '#default_value' => $settings['allow_code_visibility'],
    '#description' => t('If not allowed, all QR codes will be visible.'),
    '#weight' => 102,
  );
  
  // Allow detaching.
  $form['allow_code_detaching'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow detaching of attached QR codes'),
    '#default_value' => $settings['allow_code_detaching'],
    '#weight' => 103,
  );

  // New QR codes settings.
  $form['new'] = array(
    '#type' => 'fieldset',
    '#title' => t('New QR codes settings'),
    '#weight' => 104,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  
  // Code name.
  $form['new']['allow_code_name'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow edition of names of new QR codes'),
    '#default_value' => $settings['new']['allow_code_name'],
  );
  
  // Code description.
  $form['new']['allow_code_description'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow edition of descriptions of new QR codes'),
    '#default_value' => $settings['new']['allow_code_description'],
  );
  
  // D2C domain: <default> | <manual selection> | ...
  $domains = array();
  $domains['default'] = '- ' . t('default domain') . ' -';
  $domains['manual'] = '- ' . t('manual selection') . ' -';
  foreach(D2CDomain::usable_domains() as $domain) {
    $domains[$domain->local_id] = $domain->name . ' (' . $domain->codes_left()  . ')';
  }
  $form['new']['code_domain'] = array(
    '#type' => 'select',
    '#title' => t('D2C domain'),
    '#default_value' => $settings['new']['code_domain'],
    '#description' => t('D2C domain that will be used to create new QR codes. You can:
      (1) Use the default domain at the moment the content is created (recommended); 
      (2) Allow the manual selection of the domain when the content is created; or
      (3) Force the system to use a specific domain.'),
    '#required' => TRUE,
    '#options' => $domains,
  );
  
  // Campaign: <none> | <manual selection> | ...
  $campaigns = array();
  $campaigns['none'] = '- ' . t('none') . ' -';
  $campaigns['manual'] = '- ' . t('manual selection') . ' -';
  foreach(D2CCampaign::all() as $campaign) {
    $campaigns[$campaign->id] = $campaign->name;
  }
  $form['new']['code_campaign'] = array(
    '#type' => 'select',
    '#title' => t('Campaign'),
    '#default_value' => $settings['new']['code_campaign'],
    '#description' => t('Campaign to which new QR codes will be assigned. This is mainly for organizational 
      purposes. You can: (1) Do not assign any campaign; (2) Allow the manual selection of the campaign when the content is created; or  
      (3) Force the system to use a specific campaign.'),
    '#required' => TRUE,
    '#options' => $campaigns,
  );
  
  // Code type: <manual selection> | ...
  $code_types = array();
  $code_types['manual'] = '- ' . t('manual selection') . ' -';
  foreach(D2CAPI::code_types() as $key => $value) {
    $code_types[$key] = $value;
  }
  $code_type_description = t("For new QR codes, this determines the kind of access control that will be used
    in dot2code's servers anytime a user tries to use the code. Possible values are:");
  $code_type_description .= '<p><ul>';
  $code_type_description .= '<li><strong>' .  $code_types[D2CAPI::CODE_TYPE_ANONYMOUS] . ':</strong> ' . t('Anybody can use the code, even without a D2C user account. No information about the user will be sent to your application, as dot2code will have no way of recognising the visitor.') . '</li>';
  $code_type_description .= '<li><strong>' .  $code_types[D2CAPI::CODE_TYPE_REUSABLE] . ':</strong> ' . t('These codes require visitors to have a D2C user account to automatically retrieve valuable information about their profiles and optionally link their accounts for delegated authentication.') . '</li>';
  $code_type_description .= '<li><strong>' .  $code_types[D2CAPI::CODE_TYPE_SINGLE_USER] . ':</strong> ' . t('These codes behave like reusable codes but may only be used by a single user. The first user to capture it will invalidate the code for any other visitor, but may still use it several times himself.') . '</li>';
  $code_type_description .= '<li><strong>' .  $code_types[D2CAPI::CODE_TYPE_DISPOSABLE] . ':</strong> ' . t('More restrictive than the types before, these codes can only be used once. After a valid D2C user has captured it, the code becomes invalid.') . '</li>';
  $code_type_description .= '</ul></p>';
  $code_type_description .= t('The manual selection option allows the manual selection of the code type when the content is created.');
  $form['new']['code_type'] = array(
    '#type' => 'select',
    '#title' => t('Code type'),
    '#default_value' => $settings['new']['code_type'],
    '#description' => $code_type_description,
    '#required' => TRUE,
    '#options' => $code_types,
  );
    
  // Formatter settings.
  $form['formatter'] = array(
    '#type' => 'fieldset',
    '#title' => t('QR code formatter settings'),
    '#weight' => 105,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  
  // Show label.
  $form['formatter']['show_label'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show QR code label'),
    '#default_value' => $settings['formatter']['show_label'],
  );
  
  // Show download links.
  $form['formatter']['show_download_links'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show QR code download links'),
    '#default_value' => $settings['formatter']['show_download_links'],
  );

  // Code width.
  $form['formatter']['code_width'] = array(
    '#type' => 'textfield',
    '#title' => t('QR code width'),
    '#default_value' => $settings['formatter']['code_width'],
    '#description' => t('Leave the field empty to use the default QR code width.'),
    '#element_validate' => array('d2c_core_field_setting_validate_code_width'),
  );
  
  return $form;  
}

/**
 * 
 */
function d2c_core_field_setting_validate_code_width($element, &$form_state) {
  if (!empty($element['#value']) && !ctype_digit($element['#value'])) {
    form_error($element, t('The QR code width value should be empty or a valid integer value.'));
  }
}


/**
 * 
 * 
 * Field Type API: Widget
 *
 * 
 */


/**
 * Implements hook_field_widget_info().
 */
function d2c_core_field_widget_info() {
  return array(
    'd2c_core_code_simple_field_widget' => array(
      'label' => t('QR code'),
      'behaviors' => array(
        'multiple values' => FIELD_BEHAVIOR_DEFAULT,
        'default value' => FIELD_BEHAVIOR_NONE,
      ),
      'field types' => array('d2c_core_code'),
    ),
  );
}

/**
 * Implements hook_field_widget_form().
 */
function d2c_core_field_widget_form(&$form, &$form_state, $field, $instance, $langcode, $items, $delta, $element) {
  // Fetch item and set/recover default values.
  $default_form_values = array(
    'code_selection' => NULL,
    'code_name' => '',
    'code_description' => '',
    'code_domain' => 'default',
    'code_campaign' => 'none',
    'code_type' => D2CAPI::CODE_TYPE_REUSABLE,
    'code_detaching' => FALSE,
  ) + _d2c_core_field_generate_code_behaviour_data();
  if (isset($items[$delta])) {
    // Is the item being defined or edited in the widget (i.e. preview)?
    if (array_key_exists('d2c_code_form', $items[$delta])) {
      $default_form_values = @$items[$delta]['d2c_code_form'];
      $item = array(
        'code_id' => $items[$delta]['d2c_code_form']['code_id'],
        'label' => _d2c_core_field_get_widget_item_label($field, $instance, $items[$delta]),
        'visible' => _d2c_core_field_get_widget_item_visible($field, $instance, $items[$delta]),
      );
    }
    // Raw item from DB.
    else {
      $item = $items[$delta];
      $default_form_values['code_selection'] = $item['code_id'];
      $default_form_values = array_merge(
        $default_form_values,
        _d2c_core_field_generate_code_behaviour_data(D2CCode::find($item['code_id']))
      );
    }
  }
  else {
    // This is a new code. Set some reasonable default values.
    $item = array(
      'code_id' => 'new',
      'label' => '',
      'visible' => 1,
    );    
  }
  $is_new_code = ($item['code_id'] == 'new');
  
  // Init element.
  $element += array(
    '#delta' => $delta,
    '#element_validate' => array('d2c_core_widget_element_validate'),
  );
  $element['d2c_code_form'] = array(
    '#type' => 'fieldset',
    '#title' => ($field['cardinality'] == 1) ? $instance['label'] : NULL,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  drupal_add_js(drupal_get_path('module', 'd2c_core') . '/assets/d2c_core.js');
  drupal_add_css(drupal_get_path('module', 'd2c_core') . '/assets/d2c_core.css');
 
  switch ($instance['widget']['type']) {
    case 'd2c_core_code_simple_field_widget':

      // New code flag. This id should be used *carefully* when saving data since it can be
      // modified in the client side.
      $element['d2c_code_form']['code_id'] = array(
        '#type' => 'hidden',
        '#value' => $item['code_id'],
      );
      
      // Code creation. To be used *only* by d2c_core_field_is_empty (we don't know how to access there
      // to the instance settings)
      $element['d2c_code_form']['instance_settings_code_creation'] = array(
        '#type' => 'hidden',
        '#value' => $instance['settings']['code_creation'],
      );
      
      // Message.
      $message = NULL;
      if ($is_new_code) {
        switch ($instance['settings']['code_creation']) {
          case 'auto_entity':
            $message = t('A new QR code linking to this entity (node, user, etc.) will be created and attached to the contents.');      
            break;
          case 'auto_custom':
            $message = t("A new QR code with 'custom' behaviour will be created and attached to the contents.");      
            break;
        }
        if ($message != NULL) {
          $element['d2c_code_form']['code_creation'] = array(
            '#markup' => '<p><b>' . t($message) . '</b></p>',
          );
        }
      }
      else {
        $code = D2CCode::find($item['code_id']);
        if ($code) {
          $element['d2c_code_form']['code_summary'] = array(
            '#theme' => 'd2c_core_field_widget',
            '#code' => $code,
            '#weight' => 0,
          );
        }
      }
      
      // QR code selection.
      $behaviours = D2CCode::behaviours();
      if ($instance['settings']['code_creation'] == 'selection') {
        $codes = D2CCode::all();
        $eligible_codes = array();
        // Add -none- option only for new codes and for exiting codes when detaching is allowed.
        if (($is_new_code) ||
            ((!$is_new_code) && ($instance['settings']['allow_code_detaching']))) {
          $eligible_codes['none'] = '- ' . t('none') . ' -';
        }
        foreach($codes as $code) {
          $eligible_codes[$code->local_id] = $code->name . ' [' . $behaviours[$code->behaviour] . ']'; 
        }
        $element['d2c_code_form']['code_selection'] = array(
          '#type' => 'select',
          '#title' => t('Select an existing QR code'),
          '#default_value' => $default_form_values['code_selection'],
          '#description' => t('Select an existing code to be attached to the contents. If you are not able
            to find the QR code you need, you should <a href="@new">create it externally</a>.',
            array('@new' => url('admin/config/d2c/codes'))),
          '#options' => $eligible_codes,
        );
      }
      
      // QR code label.
      if ($instance['settings']['allow_code_label']) {
        $element['d2c_code_form']['label'] = array(
          '#title' => t('QR code label'),
          '#type' => 'textfield',
          '#default_value' => $item['label'],
          '#description' => t('A suitable label for the code.'),
        );
      }

      // QR code name.
      if (($is_new_code) &&
          (!$instance['settings']['allow_code_label']) &&
          ($instance['settings']['new']['allow_code_name']) &&
          in_array($instance['settings']['code_creation'], array('auto_entity', 'auto_custom', 'manual'))) {
        $element['d2c_code_form']['code_name'] = array(
          '#title' => t('QR code name'),
          '#type' => 'textfield',
          '#default_value' => $default_form_values['code_name'],
          '#description' => t('A suitable name for the code.'),
        );
      }
      
      // QR code description.
      if (($is_new_code) &&
          ($instance['settings']['new']['allow_code_description']) &&
          in_array($instance['settings']['code_creation'], array('auto_entity', 'auto_custom', 'manual'))) {
        $element['d2c_code_form']['code_description'] = array(
          '#title' => t('QR code description'),
          '#type' => 'textarea',
          '#default_value' => $default_form_values['code_description'],
          '#description' => t('A brief description about the code.'),
        );
      }
      
      // D2C domain.
      if (($is_new_code) &&
          ($instance['settings']['new']['code_domain'] == 'manual') &&
          in_array($instance['settings']['code_creation'], array('auto_entity', 'auto_custom', 'manual'))) {
        $domains = array();
        $domains['default'] = '- ' . t('default domain') . ' -';
        foreach(D2CDomain::usable_domains() as $domain) {
          $domains[$domain->local_id] = $domain->name . ' (' . $domain->codes_left()  . ')';
        }
        $element['d2c_code_form']['code_domain'] = array(
          '#type' => 'select',
          '#title' => t('D2C domain'),
          '#default_value' => $default_form_values['code_domain'],
          '#description' => t('D2C domain that will be used to create the code.'),
          '#options' => $domains,
        );
      }
      
      // Campaign.
      if (($is_new_code) &&
          ($instance['settings']['new']['code_campaign'] == 'manual') &&
          in_array($instance['settings']['code_creation'], array('auto_entity', 'auto_custom', 'manual'))) {
        $campaigns = array();
        $campaigns['none'] = '- ' . t('none') . ' -';
        foreach(D2CCampaign::all() as $campaign) {
          $campaigns[$campaign->id] = $campaign->name;
        }
        $element['d2c_code_form']['code_campaign'] = array(
          '#type' => 'select',
          '#title' => t('Campaign'),
          '#default_value' => $default_form_values['code_campaign'],
          '#description' => t('Campaign to which code will be assigned.'),
          '#options' => $campaigns,
        );
      }

      // QR code type.
      if (($is_new_code) &&
          ($instance['settings']['new']['code_type'] == 'manual') &&
          in_array($instance['settings']['code_creation'], array('auto_entity', 'auto_custom', 'manual'))) {
        $code_types = D2CAPI::code_types();
        $code_type_description = t('This determines the kind of access control that will be used
          in dot2code\'s servers anytime a user tries to use the code.');
        $code_type_description .= '<p><b>';
        $code_type_description .= t('Please, select the desired type carefully. It will not be possible to change it later.');
        $code_type_description .= '</b></p>';
        $element['d2c_code_form']['code_type'] = array(
          '#type' => 'select',
          '#title' => t('Code type'),
          '#default_value' => $default_form_values['code_type'],
          '#description' => $code_type_description,
          '#options' => $code_types,
        );
      }
      
      // QR code behaviour.
      if ($instance['settings']['code_creation'] == 'manual') {
        // Wrapper.
        $element['d2c_code_form']['code_behaviour']['wrapper_start'] = array(
          '#markup' => '<div class="d2c-code-behaviour-wrapper" style="padding-bottom: 15px;">',
        );

        // Selector.
        $available_behaviours = array(
          D2CCode::URLRedirectionBehaviour => $behaviours[D2CCode::URLRedirectionBehaviour],
        );
        $element['d2c_code_form']['code_behaviour']['type'] = array(
          '#type' => 'select',
          '#title' => t('Behaviour type'),
          '#description' => t('The type of behaviour the QR code will have when any user tries to
            access your application through it.'),
          '#default_value' => $default_form_values['code_behaviour']['type'],
          '#options' => $available_behaviours,
          '#attributes' => array('class' => array('d2c-code-behaviour-selection')) 
        );
      
        // D2CCode::URLRedirectionBehaviour
        $element['d2c_code_form']['code_behaviour']['params_' . D2CCode::URLRedirectionBehaviour . '_url'] = array(
          '#prefix' => '<div class="behaviour_param" id="behaviour_params_for_' . D2CCode::URLRedirectionBehaviour . '" style="display: block;">',
          '#title' => t('URL'),
          '#type' => 'textfield',
          '#default_value' => $default_form_values['code_behaviour']['params_' . D2CCode::URLRedirectionBehaviour . '_url'],
          '#suffix' => '</div>'
        );
        
        // Wrapper.
        $element['d2c_code_form']['code_behaviour']['wrapper_end'] = array(
          '#markup' => '</div>',
        );
      
      }
      
      // Visibility.
      if ($instance['settings']['allow_code_visibility']) {
        $element['d2c_code_form']['visible'] = array(
          '#type' => 'checkbox',
          '#title' => t('Visible code'),
          '#default_value' => $item['visible'],
        );
      }

      // Detaching.
      if ((!$is_new_code) &&
          ($instance['settings']['allow_code_detaching'])) {
        $element['d2c_code_form']['code_detaching'] = array(
          '#type' => 'checkbox',
          '#title' => t('Schedule code detaching'),
          '#default_value' => $default_form_values['code_detaching'],
        );
      }
      
      break;
  }
  
  return $element;
}

/**
 *
 */
function d2c_core_widget_element_validate($element, &$form_state) {
  $delta = $element['#delta'];
  $instance = field_widget_instance($element, $form_state);
  $field = field_widget_field($element, $form_state);

  // If not mandatory or unlimited cardinality skip validations, we skip validations.
  // On creation time (d2c_core_field_presave) we'll add the logic to skip non-valid and
  // empty stuff.
  if ((!$instance['required']) || ($field['cardinality'] == FIELD_CARDINALITY_UNLIMITED)) {
    return;
  }
  
  if (isset($form_state['values'][$field['field_name']][$element['#language']][$delta])) {
    $item = $form_state['values'][$field['field_name']][$element['#language']][$delta];

    $is_new_code = ($item['d2c_code_form']['code_id'] == 'new');
    
    // At this point we know the field is required and also does not have an unlimited cardinality.
    // Therefore, we perform some basic validations that would improve UX. Orther checks are
    // performed on creation time (d2c_core_field_presave).
    switch ($instance['settings']['code_creation']) {
      case 'auto_entity':
      case 'auto_custom':
        return FALSE;
        break;
      case 'manual':
        $behaviour = NULL;
        $behaviour_params = array();
        if (!_d2c_core_field_extract_widget_code_behaviour_data($item, $behaviour, $behaviour_params)) {
          form_error($element['d2c_code_form']['code_behaviour'],
                     t('Some errors were found in the behaviour parameters of the QR code.'));
        }
        break;
      case 'selection':
        if (($is_new_code) &&
            ($item['d2c_code_form']['code_selection'] == 'none')) {
          form_error($element['d2c_code_form']['code_selection'],
                     t('You must select an existing QR code to attach.'));
              
        }
        break;
    }
    
  }
}

/**
 * Implements hook_field_is_empty().
 * 
 * On fields with cardinality greater than 1, this in the function Drupal uses to check when an
 * item is supposed to be empty and when it should be processed. Due to all the
 * available cfg options in the QR code field, it is pretty hard to handle all the cases, so here we
 * add the basic logic and, on creation time (d2c_core_field_presave), we do all checks, dropping
 * items that are identified as empty on non-valid.
 */
function d2c_core_field_is_empty($item, $field) {
  if ($field['type'] == 'd2c_core_code') {
    // Is the item being defined or edited in the widget?
    if (array_key_exists('d2c_code_form', $item)) {
      $is_new_code = ($item['d2c_code_form']['code_id'] == 'new');
      switch ($item['d2c_code_form']['instance_settings_code_creation']) {
        case 'auto_entity':
        case 'auto_custom':
          return FALSE;
          break;
        case 'manual':
          if ($is_new_code) {
            $behaviour = NULL;
            $behaviour_params = array();
            _d2c_core_field_extract_widget_code_behaviour_data($item, $behaviour, $behaviour_params);
            return _d2c_core_field_empty_behaviour($behaviour, $behaviour_params);
          }
          break;
        case 'selection':
          return (($is_new_code) &&
                  ($item['d2c_code_form']['code_selection'] == 'none'));
          break;
      }
    }
    // Raw item from DB.
    else {
      return empty($item['code_id']);
    } 
  }
}

/**
 * Implements hook_field_presave().
 */
function d2c_core_field_presave($entity_type, $entity, $field, $instance, $langcode, &$items) {
  if ($field['type'] == 'd2c_core_code') {
    $nitems = count($items);
    for ($i=0; $i < $nitems; $i++) {
      // Is the item being defined or edited in the widget?
      if (array_key_exists('d2c_code_form', $items[$i])) {
      
        // New code?
        $is_new_code = ($items[$i]['d2c_code_form']['code_id'] == 'new');

        // Init the item candidate.
        $item = array(
          'code_id' => NULL,
          'label' => _d2c_core_field_get_widget_item_label($field, $instance, $items[$i], $entity_type, $entity),
          'visible' => _d2c_core_field_get_widget_item_visible($field, $instance, $items[$i], $entity_type, $entity),
          'self_reference' => FALSE, // To be used in hook_field_update/insert.
        );
    
        // Explicit detaching?
        $detaching = 
          (!$is_new_code) &&
          ($instance['settings']['allow_code_detaching']) &&
          ($items[$i]['d2c_code_form']['code_detaching']);
        
        if (!$detaching) {
          
          // Type of code creation?
          switch ($instance['settings']['code_creation']) {
            case 'auto_entity':
            case 'auto_custom':
            case 'manual':
              if ($is_new_code) {
                $code_attributtes = array(
                  'name' => '',
                  'description' => '',
                );
                
                // Name.
                $code_attributtes['name'] = $item['label'];
                if (($instance['settings']['new']['allow_code_name']) &&
                    (!empty($items[$i]['d2c_code_form']['code_name']))) {
                  $code_attributtes['name'] = $items[$i]['d2c_code_form']['code_name'];
                }
                
                // Description.
                if ($instance['settings']['new']['allow_code_description']) {
                  $code_attributtes['description'] = $items[$i]['d2c_code_form']['code_description'];
                }
                
                // Domain.
                if ($instance['settings']['new']['code_domain'] == 'manual') {
                   if ($items[$i]['d2c_code_form']['code_domain'] == 'default') {
                     $default_domain = D2CDomain::default_domain();
                     if ($default_domain) {
                       $code_attributtes['domain'] = $default_domain->local_id;
                     }
                   }
                   else {
                     $code_attributtes['domain'] = $items[$i]['d2c_code_form']['code_domain'];
                   }
                }
                elseif ($instance['settings']['new']['code_domain'] == 'default') {
                  $default_domain = D2CDomain::default_domain();
                  if ($default_domain) {
                    $code_attributtes['domain'] = $default_domain->local_id;
                  }
                }
                else {
                  $code_attributtes['domain'] = $instance['settings']['new']['code_domain'];
                }
                
                // Campaing.
                if ($instance['settings']['new']['code_campaign'] == 'manual') {
                  if ($items[$i]['d2c_code_form']['code_campaign'] != 'none') {
                    $code_attributtes['campaign'] = $items[$i]['d2c_code_form']['code_campaign'];
                  }
                }
                else {
                  if ($instance['settings']['new']['code_campaign'] != 'none') {
                    $code_attributtes['campaign'] = $instance['settings']['new']['code_campaign'];
                  }
                }
                
                // Type.
                if ($instance['settings']['new']['code_type'] == 'manual') {
                  $code_attributtes['code_type'] = $items[$i]['d2c_code_form']['code_type'];
                }
                else {
                  $code_attributtes['code_type'] = $instance['settings']['new']['code_type'];
                }
                
                // Behaviour.
                if ($instance['settings']['code_creation'] == 'manual') {
                  $behaviour = NULL;
                  $behaviour_params = array();
                  if (_d2c_core_field_extract_widget_code_behaviour_data($items[$i], $behaviour, $behaviour_params)) {
                    $code_attributtes['behaviour'] = $behaviour;
                    $code_attributtes['behaviour_params'] = $behaviour_params;
                  }
                }
                elseif ($instance['settings']['code_creation'] == 'auto_entity') {
                  // On entity editing we could set the self-reference here. Anyway, for simplicity,
                  // we always do that in the d2c_core_field_insert hook.
                  $item['self_reference'] = TRUE;
                  if ($entity_type == 'node') {
                    $code_attributtes['behaviour'] = D2CCode::NodeRedirectionBehaviour;
                    $code_attributtes['behaviour_params'] = array();
                    $code_attributtes['behaviour_params']['nid'] = 0;
                  }
                  else {
                    $code_attributtes['behaviour'] = D2CCode::URLRedirectionBehaviour;
                    $code_attributtes['behaviour_params'] = array();
                    $code_attributtes['behaviour_params']['url'] = '';
                  }
                }
                elseif ($instance['settings']['code_creation'] == 'auto_custom') {
                  // Same comment about self-references that a few lines ago :)
                  $item['self_reference'] = TRUE;
                  $code_attributtes['behaviour'] = D2CCode::CustomBehaviour;
                  $code_attributtes['behaviour_params'] = array();
                }

                // Check that we have the required minimun info to create the code. If not,
                // ignore, and it will be dropped.
                if ((isset($code_attributtes['domain'])) &&
                    (isset($code_attributtes['code_type'])) &&
                    (isset($code_attributtes['behaviour']))) {
                  $code = new D2CCode($code_attributtes);
                  if ($code->save()) {
                    $item['code_id'] = $code->local_id;
                    try {
                      $code->remote_save();
                    } catch (D2CTransportException $e) {
                      watchdog('d2c_core', 'There has been an error trying to create a QR code: ' . $e->getMessage(), NULL, WATCHDOG_ERROR);
                      drupal_set_message(t('There has been an error trying to create the QR code. The creation will be tried again later.'), 'error');
                    }
                  }
                }
              }
              // This is an existing item.
              else {
                // WARNING! We can *not* use $items[$i]['d2c_code_form']['code_id'] directly. It's ok
                // to use that value for detection of new codes, but not por existing ones. The value
                // is received from a hidden field that could has been modified in the client side.
                // We need to check that any code id received in the hidden field was also in the original
                // entity. If not, we can safely drop that shit.
                $original_items = $entity->original->{$field['field_name']}[$entity->original->language];
                $found = FALSE;
                foreach ($original_items as $original_item) {
                  if ($original_item['code_id'] == $items[$i]['d2c_code_form']['code_id']) {
                    $found = TRUE;
                    break;
                  }
                }
                if ($found) {
                  // Behaviour.
                  $code = D2CCode::find($items[$i]['d2c_code_form']['code_id']);
                  if ($code) {
                    if ($instance['settings']['code_creation'] == 'manual') {
                      $behaviour = NULL;
                      $behaviour_params = array();
                      if (_d2c_core_field_extract_widget_code_behaviour_data($items[$i], $behaviour, $behaviour_params)) {
                        $item['code_id'] = $code->local_id;
                        $code->behaviour = $behaviour;
                        $code->behaviour_params = $behaviour_params;
                        $code->save();
                      }
                      else {
                        // Implicit code detaching?
                        if ($instance['settings']['allow_code_detaching']) {
                          $detaching = TRUE;
                        }
                        // The behaviour params were not valid, but detaching codes is not
                        // alowed => Keep the id of the original code.
                        else {
                          $item['code_id'] = $items[$i]['d2c_code_form']['code_id'];
                        }
                      }
                    } 
                    else {
                      $item['code_id'] = $items[$i]['d2c_code_form']['code_id'];
                    }
                  }
                }
              }
              break;
            
            case 'selection':
              if (($items[$i]['d2c_code_form']['code_selection'] != 'none') &&
                  (!empty($items[$i]['d2c_code_form']['code_selection']))) {
                $item['code_id'] = $items[$i]['d2c_code_form']['code_selection'];
              }
              else {
                // Implicit code detaching?
                if ($instance['settings']['allow_code_detaching']) {
                  $detaching = TRUE;
                }
                // Something strange is going on... Use the id of the original attached code.
                // In this case we don't need to check the code id (see previous WARNING
                // comment).
                else {
                  $item['code_id'] = $items[$i]['d2c_code_form']['code_id'];
                }
              }
              break;
          }
          
        }
        
        // Is the item ready or should we drop this one?
        if (($detaching) || ($item['code_id'] == NULL)) {
          unset($items[$i]);
        }
        else {
          $items[$i] = $item;
        }

      }
    }
  }
}

/**
 * Implements hook_field_insert().
 * 
 *  - Behaviours of QR coded including self-references to the entity are updated here,
 *    once whe have an URI/id for the entity.
 */
function d2c_core_field_insert($entity_type, $entity, $field, $instance, $langcode, &$items) {
  if ($field['type'] == 'd2c_core_code') {
    foreach ($items as $item) {
      if ($item['self_reference']) {
        $code = D2CCode::find($item['code_id']);
        if ($code) {
          switch ($code->behaviour) {
            case D2CCode::URLRedirectionBehaviour:
              $uri = entity_uri($entity_type, $entity);
              $code->behaviour_params = array('url' => $uri['path']);
              $code->save();
              break;
            case D2CCode::NodeRedirectionBehaviour:
              list($id, $vid, $bundle) = entity_extract_ids($entity_type, $entity);
              $code->behaviour_params = array('nid' => $id);
              $code->save();
              break;
          }
        }
      }
      unset($item['self_reference']);
    }
  }  
}

/**
 * Implements hook_field_update().
 */
function d2c_core_field_update($entity_type, $entity, $field, $instance, $langcode, &$items) {
  d2c_core_field_insert($entity_type, $entity, $field, $instance, $langcode, $items);  
}

/**
 * Implements hook_field_load.
 * 
 * - Lazy filtering of removed QR codes.
 * - Retries of QR codes not remotely commited.
 * 
 * This stuff could also be implementes in hook_field_formatter_prepare_view(), but it would probably
 * be too much overhead.
 */
function d2c_core_field_load($entity_type, $entities, $field, $instances, $langcode, &$items, $age) {
  if ($field['type'] == 'd2c_core_code') {
    foreach (array_keys($items) as $id) {
      $nitems = count($items[$id]);
      for ($i=0; $i < $nitems; $i++) {
        // Is the item being defined or edited in the widget?
        if (array_key_exists('d2c_code_form', $items[$id][$i])) {
          // Go on :)
        }
        // Raw item from DB.
        else {
          $code = D2CCode::find($items[$id][$i]['code_id']);
          if ($code) {
            if ($code->status == D2CCode::StatusCommitted) {
              // Uncommitted code!
              try {
                $code->remote_save();
              } catch (D2CTransportException $e) {
                watchdog('d2c_core', 'There has been an error trying to create a QR code: ' . $e->getMessage(), NULL, WATCHDOG_ERROR);
              }
            }
          }
          else {
            // Removed code!
            unset($items[$id][$i]);
          }
        }
      }
    }
  }
}


/**
 * 
 * 
 *  Field Type API: Formatter
 *  
 *
 */


/**
 * Implements hook_field_formatter_info().
 */
function d2c_core_field_formatter_info() {
  return array(
    'd2c_core_code_simple_field_formatter' => array(
      'label' => t('QR code'),
      'field types' => array('d2c_core_code'),
    ),
  );
}

/**
 * Implements hook_field_formatter_view().
 */
function d2c_core_field_formatter_view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
  $element = array();
  switch ($display['type']) {
    case 'd2c_core_code_simple_field_formatter':
      $proccessed_items = array();
      
      // Inspect items.
      foreach ($items as $delta => $item) {
        // Is the item being defined or edited in the widget?
        if (array_key_exists('d2c_code_form', $item)) {
          $proccessed_item['code'] = D2CCode::find($item['d2c_code_form']['code_id']);
          $proccessed_item['label'] = _d2c_core_field_get_widget_item_label($field, $instance, $item, $entity_type, $entity); 
          $visible = _d2c_core_field_get_widget_item_visible($field, $instance, $item, $entity_type, $entity);
        }
        // Raw item from DB.
        else {
          $proccessed_item['code'] = D2CCode::find($item['code_id']);
          $proccessed_item['label'] = $item['label'];
          $visible = $item['visible'];
        }
        
        // Visible?
        if ($visible) {
          $proccessed_items[] = $proccessed_item;
        }
      }

      // Render items.
      drupal_add_css(drupal_get_path('module', 'd2c_core') . '/assets/d2c_core.css');
      $element[] = array (
        '#code_width' => empty($instance['settings']['formatter']['code_width']) ? d2c_core_settings_get_max_code_width() : $instance['settings']['formatter']['code_width'],
        '#show_download_links' => $instance['settings']['formatter']['show_download_links'],
        '#show_label' => $instance['settings']['formatter']['show_label'],
        '#items' => $proccessed_items,
        '#theme' => 'd2c_core_field_formatter',
      );

      break;
  }
  return $element;
}


/**
 * 
 * 
 *  Utility stuff.
 *
 *
 */


/**
 *
 */
function _d2c_core_field_get_widget_item_label($field, $instance, $item, $entity_type = NULL, $entity = NULL) {
  $result = '';
  if (($instance['settings']['allow_code_label']) &&
      (array_key_exists('label', $item['d2c_code_form']))) {
    $result = $item['d2c_code_form']['label'];
  }
  if ((empty($result)) &&
      ($instance['settings']['new']['allow_code_name']) &&
      (array_key_exists('code_name', $item['d2c_code_form']))) {
    $result = $item['d2c_code_form']['code_name'];
  }
  if (($entity) &&
      (empty($result)) &&
      (isset($entity->title))) {
    $result = $entity->title;
  }
  return $result;
}

/**
 * 
 */
function _d2c_core_field_get_widget_item_visible($field, $instance, $item, $entity_type = NULL, $entity = NULL) {
  $result = TRUE;
  if (($instance['settings']['allow_code_visibility']) &&
      (array_key_exists('visible', $item['d2c_code_form']))) {
    return $item['d2c_code_form']['visible'];
  }
  return $result;
}

/**
 * 
 */
function _d2c_core_field_generate_code_behaviour_data($code = NULL) {
  $result = array();
  $result['code_behaviour'] = array();
  $result['code_behaviour']['type'] = ($code) ? $code->behaviour : D2CCode::URLRedirectionBehaviour;
  $result['code_behaviour']['params_' . D2CCode::URLRedirectionBehaviour . '_url'] = ($code) ? @$code->behaviour_params['url'] : '';
  return $result;
}

/**
 *
 */
function _d2c_core_field_extract_widget_code_behaviour_data($item, &$behaviour, &$behaviour_params) {
  $behaviour = NULL;
  $behaviour_params = array();
  switch ($item['d2c_code_form']['code_behaviour']['type']) {
    case D2CCode::URLRedirectionBehaviour:
      $behaviour = D2CCode::URLRedirectionBehaviour;
      $key = 'params_' . $behaviour . '_url';
      $behaviour_params['url'] = (array_key_exists($key, $item['d2c_code_form']['code_behaviour'])) ? $item['d2c_code_form']['code_behaviour'][$key] : '';
      break;
  }
  return _d2c_core_field_validate_behaviour($behaviour, $behaviour_params);
}

/**
 *
 */
function _d2c_core_field_empty_behaviour($behaviour, $behaviour_params) {
  $result = FALSE;
  switch ($behaviour) {
    case D2CCode::URLRedirectionBehaviour:
      $result = empty($behaviour_params['url']);
      break;
  }
  return $result;
}

/**
 *
 */
function _d2c_core_field_validate_behaviour($behaviour, $behaviour_params) {
  $result = FALSE;
  switch ($behaviour) {
    case D2CCode::URLRedirectionBehaviour:
      $result = !empty($behaviour_params['url']);
      break;
  }
  return $result;
}
